@@ -1,4 +1,5 @@
 import json
+import sqlite3
 from typing import Protocol, Type, runtime_checkable, Self
 
 
@@ -37,3 +38,21 @@ class Model:
     def __repr__(self) -> str:
         attrs = ", ".join(f"{k}={repr(v)}" for k,v in self.__dict__.items())
         return f"{self.__class__.__name__}({attrs})"
+
+
+
+def stub(msg: str):
+    class Stub:
+        def __getattribute__(self, name: str):
+            raise TypeError(msg)
+
+        def __call__(self, *args, **kwds):
+            raise TypeError(msg)
+
+    return Stub()
+
+
+class IDatabase(Protocol):
+    @property
+    def connection(self) -> sqlite3.Connection:
+        ...
